# Specify --with gnu_prefix to prefix any GNU utilities that overlap
# with PASE utilities as gnu-, eg as -> gnu-as
# GNU-specific utils are never prefixed, however
%bcond_without gnu_prefix

%if %{with gnu_prefix}
%define bin_prefix gnu-
%else
%define bin_prefix %nil
%endif

Name: binutils-gnu
Version: 2.42
Release: 1
License: GPL-3.0-or-later
Summary: GNU binutils
Url: https://www.gnu.org/software/binutils

# Needed for makeinfo
BuildRequires: texinfo
BuildRequires: zlib-devel

Source0: https://ftp.gnu.org/pub/gnu/binutils/binutils-%{version}.tar.xz

Patch0: binutils-2.37-bfd-add-cast-to-avoid-gcc-errors.patch
Patch2: binutils-2.37-aix5l-compat.patch

# Patches not suitable to upstream
Patch100: binutils-2.37-no-native-libpath.patch

%description
The GNU Binutils are a collection of tools for building and inspecting binaries.
It primarily includes a linker (ld) and assembler (as), but also includes utilities
like ar, nm, objdump, strip, etc.

%package -n binutils-devel
Summary: BFD and opcodes static libraries and header files
Requires: %{name} = %{version}-%{release}

%description -n binutils-devel
BFD and opcodes static libraries and header files.

%prep

%setup -q -n binutils-%{version}

%patch0 -p1
%patch100 -p1
%patch2 -p1

# Binutils commit 2faf902da5109e31ad08d84a24f827f0e6f60dc4
# broke man page generation, causing them to be empty
# we've patched it above, but the man pages won't be rebuilt
# unless we remove them (or their info pages got updated somehow)
rm binutils/doc/addr2line.1
rm binutils/doc/ar.1
rm binutils/doc/c++filt.1
rm binutils/doc/dlltool.1
rm binutils/doc/elfedit.1
rm binutils/doc/nm.1
rm binutils/doc/objcopy.1
rm binutils/doc/objdump.1
rm binutils/doc/ranlib.1
rm binutils/doc/readelf.1
rm binutils/doc/size.1
rm binutils/doc/strings.1
rm binutils/doc/strip.1
rm binutils/doc/windmc.1
rm binutils/doc/windres.1
rm gas/doc/as.1
rm gprof/gprof.1
rm ld/ld.1

# This config template is used to set LDFLAGS=-Wl,-bbigtoc
# Unfortunately, this replaces our LDFLAGS setting from the command line
# so we just empty it here. Note, while we could add -bbigtoc to our
# LDFLAGS, it's not needed in binutils. The only reason it is set here is
# because this config template is shared with GCC, which does need it.
echo > config/mh-ppc-aix

%build

%define _host powerpc-ibm-aix7.1

%configure \
  ac_cv_func_utimensat=no \
  LDFLAGS="-Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib" \
  --config-cache \
  --with-system-zlib \
  --with-lib-path="%{_libdir}:/QOpenSys/usr/lib" \
  --enable-pthread \
  --enable-threads \
  --enable-64-bit-bfd \
  --enable-static \
  --disable-nls \
  --disable-rpath \
  --disable-dependency-tracking \
  #end

%make_build


%install

%make_install
rm -r %{buildroot}%{_prefix}/%{_host} || :

# These tools are only built on PE-format targets (Windows, Cygwin, ...)
# yet are always installed. No point shipping if we don't have the utility
rm %{buildroot}%{_mandir}/man1/dlltool.1
rm %{buildroot}%{_mandir}/man1/windmc.1
rm %{buildroot}%{_mandir}/man1/windres.1

# binutils doesn't enable plugins by default and on AIX it produces the warning
#   "Enabling plugins may result in ar creating non-standard archives"
# However, ld still builds the libdep plugin (and builds it as an AIX-style .a
# which plugin support can't load anyway). Since this is in the wrong format and
# can't even be used, we remove it and the empty directory
rm -r %{buildroot}%{_libdir}/bfd-plugins

%if %{with gnu_prefix}
for f in ar as ld nm ranlib size strings strip
do
  mv %{buildroot}%{_bindir}/$f %{buildroot}%{_bindir}/%{bin_prefix}$f
done
%endif

# don't ship global info dir file
rm %{buildroot}%{_infodir}/dir

%files
%defattr(-, qsys, *none)

%license COPYING
%license COPYING.LIB
%license COPYING3
%license COPYING3.LIB


%{_bindir}/%{bin_prefix}as
%{_bindir}/%{bin_prefix}ld
%{_bindir}/%{bin_prefix}ar
%{_bindir}/%{bin_prefix}nm
%{_bindir}/%{bin_prefix}ranlib
%{_bindir}/%{bin_prefix}size
%{_bindir}/%{bin_prefix}strings
%{_bindir}/%{bin_prefix}strip

%{_bindir}/ld.bfd
%{_bindir}/addr2line
%{_bindir}/c++filt
%{_bindir}/elfedit
%{_bindir}/objcopy
%{_bindir}/objdump
%{_bindir}/readelf

%{_mandir}/man1/as.1*
%{_mandir}/man1/ld.1*

%{_mandir}/man1/addr2line.1*
%{_mandir}/man1/ar.1*
%{_mandir}/man1/c++filt.1*
%{_mandir}/man1/elfedit.1*
%{_mandir}/man1/nm.1*
%{_mandir}/man1/objcopy.1*
%{_mandir}/man1/objdump.1*
%{_mandir}/man1/ranlib.1*
%{_mandir}/man1/readelf.1*
%{_mandir}/man1/size.1*
%{_mandir}/man1/strings.1*
%{_mandir}/man1/strip.1*

%{_infodir}/as.info*
%{_infodir}/bfd.info*
%{_infodir}/binutils.info*
%{_infodir}/ld.info*
%{_infodir}/ctf-spec.info*
%{_infodir}/ldint.info*
%{_infodir}/sframe-spec*

%files -n binutils-devel
%defattr(-, qsys, *none)

%{_includedir}/*

%{_libdir}/*.a

%changelog
* Fri Mar 15 2024 Abdirahim Musse <abdirahim.musse@ibm.com> - 2.42
- Update to 2.42

* Tue Mar 12 2024 Abdirahim Musse <abdirahim.musse@ibm.com> - 2.38
- Update to 2.38
- Drop upstreamed patch files
- Build require texinfo

* Fri Aug 13 2021 Kevin Adler <kadler@us.ibm.com> - 2.37
- Initial release
- GNU utilities that mask PASE are prefixed with "gnu-"
- No target path (/QOpenSys/pkgs/powerpc-ibm-aix7.1) shipped
- ld default library search path is /QOpenSys/pkgs/lib,/QOpenSys/usr/lib
- Patched to search for .so files when -brtl provided

